import { Declare, SingleData, Widget } from 'ptnl-constructor-sdk';

import {
    digitCapacityDefault,
    digitCapacityMax,
    i18N,
    numIndicatorsToShow,
} from './constants';
import { EBlockKey, EChartType, EValueType, EViewOption } from './enum';
import * as echarts from 'echarts/core';
import { LineChart, LineSeriesOption, ScatterChart } from 'echarts/charts';
import { Text } from 'ptnl-constructor-sdk/config';
import {
    TooltipComponent,
    TooltipComponentOption,
    GridComponent,
    GridComponentOption,
    LegendComponent,
    LegendComponentOption,
    MarkPointComponent,
    MarkLineComponent,
    DataZoomComponent,
    ToolboxComponent,
    TitleComponent,
} from 'echarts/components';
import { SVGRenderer } from 'echarts/renderers';
import { registerLayout, setCanvasCreator } from 'echarts/core';

echarts.use([
    TitleComponent,
    LineChart,
    ScatterChart,
    TooltipComponent,
    GridComponent,
    LegendComponent,
    MarkPointComponent,
    MarkLineComponent,
    DataZoomComponent,
    ToolboxComponent,
    SVGRenderer,
]);
type ECOption = echarts.ComposeOption<
    | LineSeriesOption
    | TooltipComponentOption
    | GridComponentOption
    | LegendComponentOption
>;
type ECharts = ReturnType<typeof echarts.init>;
const i18n: Record<string, Text> = {
    'no-value': {
        ru: 'Не найдено',
        en: 'No data',
    },
};
@Declare({
    provideCssVariables: true,
})
export class IndicatorGrad extends Widget implements SingleData {
    readonly data!: SingleData['data'];
    readonly dataSettings!: SingleData['dataSettings'];
    private planValue?: number;
    private body = document.querySelector('body');
    private diffValue?: string;
    private color = 'black';
    private indA1path?: string;
    private indB1path?: string;
    private indA1?: number;
    private indB1?: number;
    private planPath?: string;
    private addThousandsSeparator = (value: number | string): string => {
        let numberSplit = value.toString().split('.');
        numberSplit[0] = numberSplit[0].replace(
            /\B(?=(\d{3})+(?!\d))/g,
            '\u00A0',
        );
        return numberSplit.join('.');
    };

    private getValue = (value: any): string => {
        const valueFloat = parseFloat(value);

        return isNaN(valueFloat) ? '' : this.addThousandsSeparator(value);
    };

    private getValueWithDigitCapacity = (value: any): string => {
        const valueFloat = parseFloat(value);
        if (isNaN(valueFloat)) {
            return '';
        }

        let digitCapacity = parseInt(
            this.viewSettings[EViewOption.DigitCapacity],
        );
        if (!isNaN(digitCapacity)) {
            digitCapacity =
                digitCapacity < 0 || digitCapacity > digitCapacityMax
                    ? digitCapacityDefault
                    : digitCapacity;
            return this.addThousandsSeparator(
                valueFloat.toFixed(digitCapacity),
            );
        }

        return this.addThousandsSeparator(
            valueFloat.toFixed(digitCapacityDefault),
        );
    };

    private getFormattedValue = (value: any): string =>
        this.viewSettings[EViewOption.EnableDigitCapacity]
            ? this.getValueWithDigitCapacity(value)
            : this.getValue(value);
    IsJsonString(str: string) {
        try {
            JSON.parse(str);
        } catch (e) {
            return false;
        }
        return true;
    }
    applySteps(val: number) {
        if (this.viewSettings[EViewOption.EnableColorIndicator]) {
            let color_select=1;
            while(color_select<parseInt(this.viewSettings[EViewOption.IndicatorStepCount])){
                if(val<parseFloat(this.viewSettings[EViewOption.IndicatorStep+`_${color_select}`])){
                    console.log(color_select,val,parseFloat(this.viewSettings[EViewOption.IndicatorStep+`_${color_select}`]))
                    break;
                }
                color_select++;
            }
            this.color=this.viewSettings[EViewOption.IndicatorStepColor+`_${color_select}`]
        }
    }
    private getDiffParams(
        planB: number,
        planA: number,
        planA1?: number,
        planB1?: number,
    ): string {
        if (this.viewSettings.typeValue == EValueType.PlanFact) {
            const val = (planA / planB) * 100;
            let formVal = +this.getFormattedValue(val).replace(/\s/g, '');

            this.applySteps(
                this.viewSettings[EViewOption.EnableDigitCapacity]
                    ? formVal
                    : val,
            );
            return this.getFormattedValue(val);
        }
        if (this.viewSettings.typeValue == EValueType.DifferenceSegment) {
            const val = planA / planB - planA1! / planB1!;
            let formVal = +this.getFormattedValue(val).replace(/\s/g, '');

            this.applySteps(
                this.viewSettings[EViewOption.EnableDigitCapacity]
                    ? formVal
                    : val,
            );
            return this.getFormattedValue(val);
        }
        if (this.viewSettings.typeValue == EValueType.CalculatedValue) {
            const val =
                ((planA / planB - planA1! / planB1!) / (planA1! / planB1!)) *
                100;
            let formVal = +this.getFormattedValue(val).replace(/\s/g, '');

            this.applySteps(
                this.viewSettings[EViewOption.EnableDigitCapacity]
                    ? formVal
                    : val,
            );
            return this.getFormattedValue(val);
        }
        if (this.viewSettings.typeValue == EValueType.ChangeDole) {
            const val = (planA / planB) * 100 - (planA1! / planB1!) * 100;
            let formVal = +this.getFormattedValue(val).replace(/\s/g, '');

            this.applySteps(
                this.viewSettings[EViewOption.EnableDigitCapacity]
                    ? formVal
                    : val,
            );
            return this.getFormattedValue(val);
        }
        if (this.viewSettings.typeValue == EValueType.Change) {
            const val = planA - planB;
            let formVal = +this.getFormattedValue(val).replace(/\s/g, '');
            this.applySteps(
                this.viewSettings[EViewOption.EnableDigitCapacity]
                    ? formVal
                    : val,
            );
            return this.getFormattedValue(val);
        }
        if (this.viewSettings.typeValue == EValueType.ChangeInPercents) {
            const val = ((planA - planB) / planB) * 100;
            let formVal = +this.getFormattedValue(val).replace(/\s/g, '');
            this.applySteps(
                this.viewSettings[EViewOption.EnableDigitCapacity]
                    ? formVal
                    : val,
            );
            return this.getFormattedValue(val);
        }
        const val = planA / planB;
        let formVal = +this.getFormattedValue(val).replace(/\s/g, '');
        this.applySteps(
            this.viewSettings[EViewOption.EnableDigitCapacity] ? formVal : val,
        );
        return this.getFormattedValue(val);
    }

    private getScale(): void {
        const indicatorsAll = Array.from(
            document.querySelectorAll('.indicator'),
        );
        const container = document.querySelector(
            '.container',
        ) as HTMLDivElement;
        let main = document.querySelector('.main') as HTMLDivElement;
        let value = document.querySelector('.value') as HTMLDivElement;
        let text = document.querySelector('.text') as HTMLDivElement;
        let scaleHeight = 0;
        let heightBlock = text.clientHeight + value.clientHeight - 10;
        // functions ///////////////////////////////////////////////////////////////////////////
        function setScale(scale: number): void {
            container.style.setProperty('transform', `scale(${scale})`);
        }
        function setOrigin(position: string): void {
            container.style.setProperty('transform-origin', position);
        }
        function removeProp(): void {
            container.style.removeProperty('transform');
            container.style.removeProperty('transform-origin');
        }
        function absValue(
            textWidth: number,
            scaleHeight: number,
            bodyWidth: number,
        ): string {
            return (
                Math.abs(100 - ((textWidth * scaleHeight) / bodyWidth) * 100) +
                '% 0'
            );
        }
        /////////////////////////////////////////////////////////////////////////////////////
        if (document.body.clientHeight <= 231) {
            scaleHeight = 0;
            if (document.body.clientHeight <= 151) {
                if (heightBlock > 109) {
                    scaleHeight = 109 / heightBlock;
                    setScale(scaleHeight);
                    if (document.body.clientWidth < 180)
                        setOrigin(
                            absValue(
                                text.clientWidth,
                                scaleHeight,
                                document.body.clientWidth,
                            ),
                        );
                    else setOrigin('top');
                }
            } else if (document.body.clientHeight <= 171) {
                if (heightBlock > 159) {
                    scaleHeight = 159 / heightBlock;
                    setScale(scaleHeight);
                    if (document.body.clientWidth < 180)
                        setOrigin(
                            absValue(
                                text.clientWidth,
                                scaleHeight,
                                document.body.clientWidth,
                            ),
                        );
                    else setOrigin('top');
                }
            } else if (heightBlock > 231) {
                scaleHeight = 231 / heightBlock;
                setScale(scaleHeight);
                setOrigin('top');
            }
            if (scaleHeight == 0) {
                removeProp();
            }
        } else {
            removeProp();
        }

        if (indicatorsAll.length > 0) {
            const indicatorsToShow = indicatorsAll.slice(
                0,
                numIndicatorsToShow,
            );

            const width = indicatorsToShow.reduce(
                (acc, item) => acc + item.clientWidth,
                0,
            );
            const containerWidth = window.innerWidth;
            const scale = containerWidth / width;
            if (scaleHeight != 0) {
                if (scaleHeight > scale && scale < 1) {
                    setScale(scale);
                    setOrigin('0 0');
                } else if (scaleHeight > 1) {
                    removeProp();
                }
            } else {
                if (scale < 1) {
                    setScale(scale);
                    setOrigin('0 0');
                } else {
                    removeProp();
                }
            }
        }
    }

    private renderWidget(): void {
        const container = document.getElementById('root');
        if (this.data.length > 0) {
            const data: {
                [path: string]: number;
            } = this.data[0];

            const isPlanExist =
                this.dataSettings.columnsByBlock[EBlockKey.IndicatorA].length >
                0;
            const isShowFactName = this.viewSettings[EViewOption.ShowFactName];

            const indicators = this.dataSettings.columnsByBlock[
                EBlockKey.IndicatorA
            ].map((valueItem) => {
                const factName = valueItem.name;
                const factPath = valueItem.path;
                const factValue = data[factPath];

                const nameDiv = isShowFactName
                    ? `<div class="name">${factName}</div>`
                    : '';

                let ValueHorizontal = '';
                let valueVertical = '';
                let indA = 0;
                if (isPlanExist) {
                    if (
                        this.dataSettings.columnsByBlock[EBlockKey.IndicatorB]
                            .length > 0
                    ) {
                        this.planPath =
                            this.dataSettings.columnsByBlock[
                                EBlockKey.IndicatorB
                            ][0].path;
                        if (
                            this.dataSettings.columnsByBlock[
                                EBlockKey.IndicatorA1
                            ].length != 0
                        ) {
                            this.indA1path =
                                this.dataSettings.columnsByBlock[
                                    EBlockKey.IndicatorA1
                                ][0].path;
                            this.indA1 = data[this.indA1path];
                        }
                        if (
                            this.dataSettings.columnsByBlock[
                                EBlockKey.IndicatorA1
                            ].length != 0
                        ) {
                            this.indB1path =
                                this.dataSettings.columnsByBlock[
                                    EBlockKey.IndicatorB1
                                ][0].path;
                            this.indB1 = data[this.indB1path];
                        }
                        this.planValue = data[this.planPath];
                        if (
                            this.indA1 != undefined &&
                            this.indB1 != undefined
                        ) {
                            this.diffValue = this.getDiffParams(
                                this.planValue!,
                                factValue,
                                this.indA1,
                                this.indB1,
                            );
                        } else
                            this.diffValue = this.getDiffParams(
                                this.planValue!,
                                factValue,
                            );
                    } else {
                        this.applySteps(factValue);
                        indA = factValue;
                    }
                    ValueHorizontal = `
                    <div class="data__wrapper" ${
                        isShowFactName
                            ? 'style= "grid-template-columns: 1fr 1fr;"'
                            : ''
                    }>
                    ${
                        isShowFactName
                            ? `<div class="data__left">
                            <div class="indicator1" style="font-size: ${
                                this.viewSettings[EViewOption.FontSizeName]
                            }px; font-weight:bold"">
                                ${this.viewSettings[EViewOption.Name]}
                            </div>
                            
                            <div class="indicator2" style="font-size: ${
                                this.viewSettings[
                                    EViewOption.FontSizeDescription
                                ]
                            }px; color: #6f6f78"">
                                ${this.viewSettings[EViewOption.Description]}
                            </div>
                            <div class="data__fact-value 
                            }">
                            </div>
                        </div>`
                            : ''
                    }
                        <div style="font-size: ${
                            this.viewSettings[EViewOption.FontSizeIndex]
                        }px; color: ${
                        this.viewSettings[EViewOption.EnableColorIndicator]
                            ? this.color
                            : this.body?.className.includes(
                                  'ptnl-theme-dark',
                              ) == true
                            ? '#e5e7eb'
                            : 'black'
                    }" class="data__right" >
                            ${
                                indA != 0
                                    ? this.getFormattedValue(indA) == undefined
                                        ? ''
                                        : this.getFormattedValue(indA)
                                    : this.diffValue == undefined
                                    ? ''
                                    : this.diffValue
                            }${this.viewSettings[EViewOption.Suffix]}
                        </div>
                    </div>`;

                    valueVertical = `
                    <div class="main">
                    ${
                        isShowFactName
                            ? `
                        <div class="text">
                            <div class="title" style="font-size: ${
                                this.viewSettings[EViewOption.FontSizeName]
                            }px; font-weight:bold"">
                                ${this.viewSettings[EViewOption.Name]}
                            </div>
                            <div class="description" style="font-size: ${
                                this.viewSettings[
                                    EViewOption.FontSizeDescription
                                ]
                            }px; color: #6f6f78"">
                                ${this.viewSettings[EViewOption.Description]}
                            </div>
                        </div>`
                            : ''
                    }
                        <div style="font-size: ${
                            this.viewSettings[EViewOption.FontSizeIndex]
                        }px; font-weight:bold; color: ${
                        this.viewSettings[EViewOption.EnableColorIndicator]
                            ? this.color + ';'
                            : this.body?.className.includes(
                                  'ptnl-theme-dark',
                              ) == true
                            ? '#e5e7eb'
                            : 'black;'
                    }  ${
                        !isShowFactName ? 'margin-top: auto' : ''
                    }" class="value" >
                    <span>${
                        indA != 0
                            ? this.getFormattedValue(indA)
                            : this.diffValue
                    }${this.viewSettings[EViewOption.Suffix]}
                    </span>
                        </div>
                    </div>`;
                } else {
                    ValueHorizontal = `<span class="data__no-plan">${this.getFormattedValue(
                        factValue,
                    )}</span>`;
                }

                return this.viewSettings[EViewOption.ChartType] ==
                    EChartType.HorizonatalType
                    ? `
            <div class="indicator" style="display: flex;
            flex-direction: column;
            margin: auto;
            align-items: center;
            padding: 0.5rem 1rem;
            width: auto;">
                ${ValueHorizontal}
            </div>
            `
                    : `
                <div class="indicator" style="height: 100%">
                    
                    ${valueVertical}
                </div>
                `;
            });

            (container as HTMLDivElement).innerHTML =
                this.viewSettings[EViewOption.ChartType] ==
                EChartType.HorizonatalType
                    ? `<div class="container" style="display: flex;
                    flex-direction: row;
                    flex-wrap: wrap;">${indicators.join('')}</div>`
                    : `<div class="container" style="height: 100%">${indicators.join(
                          '',
                      )}</div>`;
            (container as HTMLDivElement).style.setProperty('height', '100vh');

            this.getScale();
        } else {
            (container as HTMLDivElement).innerHTML = '';
        }
    }

    onLangChange(): void {
        this.onChange();
    }

    onThemeChange(): void {}

    onChange(): void {
        this.renderWidget();
        this.ready();
    }
    onInit(): void {
        window.removeEventListener('resize', this.getScale);
        window.addEventListener('resize', this.getScale);
    }
}
